'use strict';
// Ionic Starter App, v0.9.20

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.services' is found in services.js
// 'starter.controllers' is found in controllers.js
var socket = io.connect('http://tianguisapp.herokuapp.com:80');
angular.module('Tianguisapp', ['ionic', 'Tianguisapp.controllers','wu.masonry'])

.run(function($ionicPlatform) {
  $ionicPlatform.ready(function() {
    if(window.StatusBar) {
      StatusBar.styleDefault();
    }
  });

  // Prototipo de localStorage  para guardar objectos
  Storage.prototype.setObject = function(key, value) {
    this.setItem(key, JSON.stringify(value));
  };

  // Prototipo de localStorage  para obtener objectos
  Storage.prototype.getObject = function(key) {
    var value = this.getItem(key);
    return value && JSON.parse(value);
  };
})


.factory('socket', function ($rootScope, $location) {

  return {
    on: function (eventName, callback) {
      socket.on(eventName, function () {
        var args = arguments;
        $rootScope.$apply(function () {
          callback.apply(socket, args);
        })
      })
    },
    emit: function (eventName, data, callback) {
      socket.emit(eventName, data, function () {
        var args = arguments;
        $rootScope.$apply(function () {
          callback.apply(socket, args);
        })
      })
    }
  };

})

.factory('Geo', function ($rootScope) {
  var timesOpen = 0;
  window.point = { latitude: 20.653467, longitude: -105.225255 };


  return {
    point : function (cb) {
      if (!navigator.geolocation) {
        return window.point;
      }
      document.addEventListener("deviceready", function(){
        navigator.geolocation.getCurrentPosition(function(position) {
          var gps = {
            latitude: position.coords.latitude,
            longitude: position.coords.longitude
          };
          window.point = gps;
          cb(gps);
        },function (error) {
          cb(window.point);
        },{ timeout: 3000 });
      }, false);

    },
    addTime: function () {
      timesOpen = timesOpen+1;
      console.log(timesOpen);
    },
    getTimesOpen: function () {
      return timesOpen;
    }
  }

})

.filter('range', function() {
  return function(input, total) {
    total = parseInt(total);
    for (var i=0; i<total; i++)
      input.push(i);
    return input;
  };
})

.directive('customOnChange', function() {
  return {
    restrict: 'A',
    link: function (scope, element, attrs) {
      var onChangeFunc = element.scope()[attrs.customOnChange];
      element.bind('change', onChangeFunc);
    }
  };
})

.service('User',function ($rootScope) {
  var urlTarget = '';
  var productCategories;
  loadCategories(function () {
  });
  return {
    isLogged: function (cb) {
      $.ajax({
        url: 'http://tianguisapp.herokuapp.com/connect',
        type: 'GET',
        contentType: false,
        cache: true,
        processData:false,
        async: false,
        xhrFields: {
          withCredentials: true
        },
        crossDomain: true,
        dataType: 'json',
        success: function (data) {
          cb(data.status === 'OK');
        },
        error: function (err) {
          cb(false);
        }
      });
    },
    isSaved: function () {
      var login = localStorage.getObject('login');
      console.log('login',login);
      if ($.isEmptyObject(login)) {
        return false;
      }else{
        return (login.email!==null && ( login.pass!==null || login.token!==null ) );
      }
    },
    setPendingUrl: function (url) {
      urlTarget = url;
    },
    getInfo: function () {
      try{
        if (!$.isEmptyObject(localStorage.getObject('user'))) {
          return localStorage.getObject('user');
        }else{
          return false;
        }
      }catch (err){
        return false;
      }
    },
    setInfo: function (user) {
      localStorage.setObject('user',user);
      return true;
    },
    getPendingUrl: function () {
      return urlTarget;
    },
    login: function (login,coords, callback) {
      var email = login.email;
      var pass  = login.password;
      var url = 'http://tianguisapp.herokuapp.com/';

      if (login.facebook) {
        url = url+'login-facebook';
      }else{
        url = url+'login';
      }

      localStorage.clear();
      console.log(login);
      if ( email.trim()==='' ) {
        callback( 'El email no es válido');
      }

      if (pass.trim()==='') {
        callback( 'La constraseña no es válida');
      }

      var form     = login;
      var formData = new FormData();
      formData.append('latitude',coords.latitude);
      formData.append('longitude',coords.longitude);
      formData.append('email',email);
      formData.append('password',pass);

      $.ajax({
        url: url,
        type: 'POST',
        data:  formData,
        contentType: false,
        cache: true,
        processData:false,
        async: false,
        xhrFields: {
          withCredentials: true
        },
        crossDomain: true,
        success: function (data) {
          if (data.error) {
            // Muestra el el error correspondiente
            callback( 'El email o contraseña son incorrectos');
          }else{
            // Guardar Usuario en localStorage
            f_getChats(function (chats, users) {
              $rootScope.$broadcast('chat-list', users);
            });
            // Se identifica con _id de usuario
            socket.emit('join', {_id:data._id});
            // Guarda variables en localstorage
            localStorage.setObject('user',data);
            localStorage.setObject('login',{password:pass,email:email});
            // Registra un nuevo Id para notificaciones
            notifications_register();
            callback( false);
          }

        },
        error: function (err) {
          console.log('error',err);
          callback( 'No hay conexión a internet');
        }
      });
    },
    getProductCategories: function (cb) {
      if (!productCategories) {
        loadCategories(cb);
      }else{
        cb(productCategories);
      }
    }
  };

  function loadCategories (cb) {
    if (!productCategories) {
      $.ajax({
        url: 'http://tianguisapp.herokuapp.com/categories',
        type: 'GET',
        contentType: false,
        cache: true,
        processData:false,
        xhrFields: {
          withCredentials: true
        },
        crossDomain: true,
        dataType: 'json',
        success: function (data) {
          cb(data.categories);
          productCategories = data.categories;
        },
        error: function (data) {
          console.log('error',data);
          cb();
        }
      });
    }
  }

})

.service('Chat', function ($rootScope) {
  var currentMsg = '';
  if (!localStorage.getObject("contact-list")) {
    localStorage.setObject("contact-list",{users:[]});
  }
  return {
    setTmpMessage: function (msg) {
      currentMsg = msg;
    },
    getTmpMessage: function () {
      console.log(currentMsg);
      return currentMsg;
    },
    getChats: function (cb) {
      return f_getChats(cb);
    }
  };
})

.service('formService', function () {
  var form = null ;
  return {
    set : function(f) {
      form = f;
    },
    get : function() {
      return form;
    }
  };
})

.config(function($stateProvider, $urlRouterProvider) {
  $stateProvider
    .state('app', {
      url: '/',
      abstract: true,
      templateUrl: 'templates/menu.html',
      controller: 'AppCtrl'
    })

    .state('app.products', {
      url: 'products',
      views: {
        'menuContent' :{
          templateUrl: 'templates/products.html',
          controller: 'ProductsCtrl'
        }
      }
    })

    .state('app.products-request', {
      url: 'products/user/:userid',
      views: {
        'menuContent' :{
          templateUrl: 'templates/product-request.html',
          controller: 'ProductsRequestCtrl'
        }
      }
    })

    .state('app.products-category-request', {
      url: 'products/category/:categoryid',
      views: {
        'menuContent' :{
          templateUrl: 'templates/product-request.html',
          controller: 'ProductsRequestCtrl'
        }
      }
    })


    .state('app.MyProducts', {
      url: 'my-products',
      views: {
        'menuContent' :{
          templateUrl: 'templates/my-products.html',
          controller: 'MyProductsCtrl'
        }
      }
    })

    .state('app.product-detail', {
      url: 'product/:id',
      views: {
        'menuContent' :{
          templateUrl: 'templates/product-detail.html',
          controller: 'ProductCtrl'
        }
      }
    })

    .state('app.config', {
      url: 'config',
      views: {
        'menuContent' :{
          templateUrl: 'templates/config.html',
          controller: 'ConfigCtrl'
        }
      }
    })

    .state('app.chating', {
      url: 'chat/:id',
      views: {
        'menuContent' :{
          templateUrl: 'templates/chating.html',
          controller: 'ChatCtrl'
        }
      }
    })

    .state('app.newProducto', {
      url: 'new_product',
      views: {
        'menuContent' :{
          templateUrl: 'templates/new-product.html',
          controller: 'NewProductCtrl'
        }
      }
    })

    .state('app.singup', {
      url: 'singup',
      views: {
        'menuContent' :{
          templateUrl: 'templates/singup.html',
          controller: 'SingupCtrl'
        }
      }
    })

    .state('app.singuplocation', {
      url: 'singuplocation',
      views: {
        'menuContent' :{
          templateUrl: 'templates/singup-location.html',
          controller: 'SinguplocationCtrl'
        }
      }
    })

    .state('app.login', {
      url: 'login',
      views: {
        'menuContent' :{
          templateUrl: 'templates/login.html',
          controller: 'LoginCtrl'
        }
      }
    })

    .state('app.loginIntro', {
      url: 'login-intro',
      views: {
        'menuContent' :{
          templateUrl: 'templates/login-intro.html',
          controller: 'LoginIntroCtrl'
        }
      }
    })

    .state('app.fullProductImage', {
      url: 'photo/product/:productId/:imageNum',
      views: {
        'menuContent' :{
          templateUrl: 'templates/full-image.html',
          controller: 'ProductImageFullCtrl'
        }
      }
    })

    .state('app.noConnection', {
      url: 'no-connection',
      views: {
        'menuContent' :{
          templateUrl: 'templates/no-connection.html'
        }
      }
    });

  // if none of the above states are matched, use this as the fallback
  $urlRouterProvider.otherwise('products');
});

function f_getChats (cb) {
  $.ajax({
    url: 'http://tianguisapp.herokuapp.com/chat/list',
    type: 'GET',
    contentType: false,
    cache: true,
    processData:false,
    xhrFields: {
      withCredentials: true
    },
    crossDomain: true,
    dataType: 'json',
    success: function (data) {
      if (data.status === 'OK') {
        cb(data.chats, data.users);
      }else{
        return;
      }
    },
    error: function (data) {
      console.log(data.responseText);
      return;
    }
  });
}